<?php

namespace Authstack\OpenIDConnect\Model\Protocol\Flow;

class Implicit implements FlowInterface
{
    protected function getResponseType(): array
    {
        return [
            'response_type' => 'token'
        ];
    }
}